from django.shortcuts import render, redirect, get_object_or_404
from .forms import RegisterForm
from mri_users.helpers import mail_with_template, user_to_ldap,\
    sync_one_uid, delete_ldap_user, change_ldap_user_data, restore_user_from_trash,\
    change_user_passwd
from mri_users.models import NewUser, ChangePasswdLink, WaitingValidationMails,\
    UserTrash, MailsModels
from mri_links.models import LiensOutils
from django.contrib.auth import authenticate, login, logout
from django.urls.base import reverse
from django.views.decorators.http import require_POST
from .models import User
from django.core.paginator import Paginator
from mri_users.forms import SearchUser, ManageMyAccountForm,\
    RefuseRegister, ManageAccountForm, PasswordChangeForm, ChangePasswdLinkForm,\
    SetPasswordForm, LockAccountForm, MailsModelsForm
from django.db.models import Q
from django.contrib.auth.decorators import login_required, user_passes_test
from django.views import View
from django.contrib.auth.mixins import LoginRequiredMixin, UserPassesTestMixin
from MRIOutils import settings
from django.apps import apps
from django.forms.models import model_to_dict
from uuid import uuid4
from django.contrib import messages
from django.utils.translation import gettext_lazy as _


if apps.is_installed('mri_db_logger'):
    from mri_db_logger.models import ActionsLogs


class AdministratorMixin(UserPassesTestMixin):
    def test_func(self):
        if self.request.user.account_type == 'administrator':
            return True
        return False
    
def user_is_admin(user):
    if user.account_type == 'administrator':
        return True
    return False

def homepage(request):
    '''
    Page d'acceuil de l'outils
    '''
    # genere les liens à afficher
    liens = LiensOutils.objects.all()
    return render(request, "homepage/homepage.html", {"liens": liens})

def register(request):
    '''
    Page inscription
    '''
    if request.method == "POST":
        form = RegisterForm(request.POST)
        if form.is_valid():
            utilisateur = form.save(commit=False)
            utilisateur.is_active = False
            utilisateur.save()
            mail_with_template('mails/mail_confirmation.html', utilisateur.email,
                              request=request, newuser=utilisateur)
            if apps.is_installed('mri_db_logger'):
                infos = model_to_dict(utilisateur, ['username', 'first_name', 'last_name', 'email', 'account_type'])
                ActionsLogs.objects.create(action='Register',
                                           infos=infos)
            return redirect(reverse('succes'))
    else:
        form = RegisterForm()

    return render(request, "register/register.html", {"form":form})

def register_succes(request):
    '''
    Page si inscription ok
    '''
    return render(request, "succes.html")

def confirm_mail(request, uuid):
    '''
    Page de confirmation du mail apres inscription
    '''
    def valid_register_mail():
        # validation du mail inscription
        utilisateur = NewUser.objects.get(email_confirmation_uuid=uuid)
        if utilisateur.email_confirmed:
            return redirect(reverse('homepage'))
        utilisateur.email_confirmed = True
        utilisateur.save()
        mail_with_template('mails/new_user.html', settings.EMAIL_DEST_ADM,
                           request=request, newuser=utilisateur)
        mail_with_template('mails/confirm_creation_et_login.html', utilisateur.email,
                           request=request, newuser=utilisateur)
        if apps.is_installed('mri_db_logger'):
            infos = model_to_dict(utilisateur, ['username', 'first_name', 'last_name', 'email', 'account_type'])
            ActionsLogs.objects.create(action='Confirm mail',
                                       infos=infos)
        return render(request, "register/confirm_mail.html")

    def valid_change_mail():
        waiting_validation_mail = WaitingValidationMails.objects.get(email_confirmation_uuid=uuid)
        if waiting_validation_mail.email_confirmed:
            return redirect(reverse('homepage'))
        waiting_validation_mail.email_confirmed = True
        waiting_validation_mail.save()
        mail_with_template('mails/mail_change_waiting_validation.html', settings.EMAIL_DEST_ADM,
                           request=request, user=waiting_validation_mail.user)
        mail_with_template('mails/confirm_mail_change_waiting_validation.html', waiting_validation_mail.new_email,
                           request=request)
        if apps.is_installed('mri_db_logger'):
            infos = {'new_email': waiting_validation_mail.new_email,
                     'username': waiting_validation_mail.user.username}
            ActionsLogs.objects.create(action='Confirm mail on change mail',
                                       infos=infos)
        return render(request, "manage_user/confirm_mail_on_change.html")

    if NewUser.objects.filter(email_confirmation_uuid=uuid).exists():
        return valid_register_mail()
    elif WaitingValidationMails.objects.filter(email_confirmation_uuid=uuid).exists():
        return valid_change_mail()
    else:
        return render(request, "register/invalid_or_used_link.html")

@require_POST
def login_view(request):
    '''
    Page login utilisateur frontend
    '''
    username = request.POST['username']
    password = request.POST['password']
    user = authenticate(request, username=username, password=password)
    if user is not None:
        login(request, user)
        return redirect(reverse('homepage'))
    return render(request, "fail_mdp.html")

@login_required
def logout_view(request):
    '''
    Logout
    '''
    logout(request)
    return redirect(reverse('homepage'))

class ManageUser(LoginRequiredMixin, AdministratorMixin, View):
    template_name = 'manage_user/manage_user.html'
    http_method_names = ['get', 'post']

    def get(self, request, *args, **kwargs):
        search_form = SearchUser()
        search = [False, ]
        liste_user = User.objects.all()
        liste_user_paginator = Paginator(liste_user, 25)
        page_number = request.GET.get('page')
        liste_user_page = liste_user_paginator.get_page(page_number)
        new_user = NewUser.objects.filter(email_confirmed=True)
        new_user_mail_not_confirmed = NewUser.objects.filter(email_confirmed=False)
        waiting_validation_mails = WaitingValidationMails.objects.all()
        form_refuse = RefuseRegister()
        show_trash = UserTrash.objects.all().count()
        return render(request, self.template_name, {
            "liste_user": liste_user_page,
            "search_form": search_form,
            "search": search,
            "new_user": new_user,
            "new_user_mail_not_confirmed": new_user_mail_not_confirmed,
            "waiting_validation_mails": waiting_validation_mails,
            'form_refuse': form_refuse,
            'show_trash': show_trash})

    def post(self, request, *args, **kwargs):
        search_form = SearchUser(request.POST)
        if search_form.is_valid():
            liste_user = User.objects.filter(
                Q(username__icontains=search_form.cleaned_data['search']) |
                Q(first_name__icontains=search_form.cleaned_data['search']) |
                Q(last_name__icontains=search_form.cleaned_data['search']))[:25]
            search = [True, search_form.cleaned_data['search'].lower()]
        else:
            search = [False, ]
            liste_user_all = User.objects.all()
            liste_user_paginator = Paginator(liste_user_all, 25)
            page_number = request.GET.get('page')
            liste_user = liste_user_paginator.get_page(page_number)
        new_user = NewUser.objects.filter(email_confirmed=True)
        return render(request, self.template_name, {
            "liste_user": liste_user,
            "search_form": search_form,
            "search": search,
            "new_user": new_user})


class Corbeille(LoginRequiredMixin, AdministratorMixin, View):
    template_name = 'manage_user/corbeille.html'
    http_method_names = ['get']

    def get(self, request, *args, **kwargs):
        liste_user = UserTrash.objects.all()
        return render(request, self.template_name, {
            "liste_user": liste_user})


@login_required
@user_passes_test(user_is_admin)
def vider_corbeille(request):
    """
    Vide la table de la corbeille
    """
    if apps.is_installed('mri_db_logger'):
        ActionsLogs.objects.create(action='Empty Recycle Bin',
                                   user=request.user,
                                   infos="Corbeille utilisateur vidé")
    UserTrash.objects.all().delete()
    return redirect(reverse('manage_user'))

@login_required
@user_passes_test(user_is_admin)
def restore_user(request, user_trash_pk):
    """
    Restore depuis la corbeille
    """
    user = get_object_or_404(UserTrash, pk=user_trash_pk)
    if apps.is_installed('mri_db_logger'):
        ActionsLogs.objects.create(action='Restore user',
                                   user=request.user,
                                   infos="Restoration de l'utilisateur {}".format(
                                       user.username))
    restore_user_from_trash(user)
    return redirect(reverse('corbeille'))


@require_POST
@login_required
@user_passes_test(user_is_admin)
def resgister_refuse_with_reason(request, user_pk):
    """
    Refuse l'inscription avec raison
    """
    form = RefuseRegister(request.POST)
    if form.is_valid():
        utilisateur = get_object_or_404(NewUser, pk=user_pk)
        mail_with_template('mails/register_refuse.html', utilisateur.email,
                          request=request, newuser=utilisateur,
                          raison=form.cleaned_data['raison'])
        if apps.is_installed('mri_db_logger'):
            infos = model_to_dict(utilisateur, ['username', 'first_name', 'last_name', 'email', 'account_type'])
            ActionsLogs.objects.create(action='Register refuse',
                                       user=request.user,
                                       infos={'raison': form.cleaned_data['raison'],
                                              'utilisateur': infos})
        utilisateur.delete()
    return redirect(reverse('manage_user'))

@login_required
@user_passes_test(user_is_admin)
def register_delete_unvalidated_mail(request, user_pk):
    """
    Supprime une inscription avec mail non validé
    """
    utilisateur = get_object_or_404(NewUser, pk=user_pk)
    if apps.is_installed('mri_db_logger'):
        infos = model_to_dict(utilisateur, ['username', 'first_name', 'last_name', 'email', 'account_type'])
        ActionsLogs.objects.create(action='Delete register with unvalidated mail',
                                   user=request.user,
                                   infos={'utilisateur': infos})
    utilisateur.delete()
    return redirect(reverse('manage_user'))

@login_required
@user_passes_test(user_is_admin)
def delete_user(request, user_pk):
    """
    Supprime un compte utilisateur
    """
    utilisateur = get_object_or_404(User, pk=user_pk)
    if apps.is_installed('mri_db_logger'):
        infos = model_to_dict(utilisateur, ['username', 'first_name', 'last_name', 'email', 'account_type'])
        ActionsLogs.objects.create(action='Delete user',
                                   user=request.user,
                                   infos=infos)
    delete_ldap_user(utilisateur)
    return redirect(reverse('manage_user'))

@login_required
@user_passes_test(user_is_admin)
def register_accept(request, user_pk):
    """
    Finalise l'inscription
    """
    newuser = get_object_or_404(NewUser, pk=user_pk)
    username = newuser.username
    user_to_ldap(newuser)
    utilisateur = User.objects.get(username=username)
    if apps.is_installed('mri_db_logger'):
        infos = model_to_dict(utilisateur, ['username', 'first_name', 'last_name', 'email', 'account_type'])
        ActionsLogs.objects.create(action='Register accept',
                                   user=request.user,
                                   infos=infos)
    mail_with_template('mails/register_accept.html', utilisateur.email,
                          request=request, newuser=utilisateur)
    return redirect(reverse('manage_user'))

@login_required
@user_passes_test(user_is_admin)
def mail_change_accept(request, new_mail_pk):
    """
    Finalise le changement de mail
    """
    new_mail_model = WaitingValidationMails.objects.get(pk=new_mail_pk)
    username = new_mail_model.user.username
    old = {'mail': [new_mail_model.user.email.encode('utf-8')]}
    new = {'mail': [new_mail_model.new_email.encode('utf-8')]}
    change_ldap_user_data(new_mail_model.user, old, new)
    if apps.is_installed('mri_db_logger'):
        infos = {'username': new_mail_model.user.username,
                 'old_mail': new_mail_model.user.email,
                 'new_mail': new_mail_model.new_email}
        ActionsLogs.objects.create(action='Mail change accept',
                                   user=request.user,
                                   infos=infos)
    mail_with_template('mails/change_mail_accept.html',
                       new_mail_model.new_email,
                       request=request)
    new_mail_model.delete()
    sync_one_uid(username)
    return redirect(reverse('manage_user'))

@require_POST
@login_required
@user_passes_test(user_is_admin)
def mail_change_refuse_with_reason(request, new_mail_pk):
    """
    Refuse le changement de mail avec raison
    """
    form = RefuseRegister(request.POST)
    if form.is_valid():
        new_mail_model = WaitingValidationMails.objects.get(pk=new_mail_pk)
        mail_with_template('mails/change_mail_refuse.html',
                           [new_mail_model.new_email, new_mail_model.user.email],
                           raison=form.cleaned_data['raison'])
        if apps.is_installed('mri_db_logger'):
            infos = {'username': new_mail_model.user.username,
                     'old_mail': new_mail_model.user.email,
                     'new_mail': new_mail_model.new_email}
            ActionsLogs.objects.create(action='Register refuse',
                                       user=request.user,
                                       infos={'raison': form.cleaned_data['raison'],
                                              'data': infos})
        new_mail_model.delete()
    return redirect(reverse('manage_user'))


class ManageAccount(LoginRequiredMixin, AdministratorMixin, View):
    template_name = 'manage_user/manage_account.html'
    http_method_names = ['get', 'post']

    def get(self, request, *args, **kwargs):
        user = get_object_or_404(User, pk=self.kwargs['user'])
        sync_one_uid(user.username)
        user.refresh_from_db()
        manage_user_form = ManageAccountForm(instance=user, request=request)
        return render(request, self.template_name, {
            "manage_user_form": manage_user_form,
            "form_refuse": LockAccountForm()})

    def post(self, request, *args, **kwargs):
        user = get_object_or_404(User, pk=self.kwargs['user'])
        before = model_to_dict(user, ['username', 'first_name', 'last_name', 'email', 'account_type'])
        manage_user_form = ManageAccountForm(request.POST, instance=user, request=request)
        saved = False
        if manage_user_form.has_changed() and manage_user_form.is_valid():
            saved = True
            manage_user_form.save()
            user.refresh_from_db()
            after = model_to_dict(user, ['username', 'first_name', 'last_name', 'email', 'account_type'])
            if apps.is_installed('mri_db_logger'):
                ActionsLogs.objects.create(action='Manage account',
                                           user=request.user,
                                           infos={'before': before,
                                                  'after': after})
        return render(request, self.template_name, {
            "manage_user_form": manage_user_form,
            "form_refuse": LockAccountForm(),
            "saved": saved})


@require_POST
@login_required
@user_passes_test(user_is_admin)
def lock_account(request, user):
    """
    Verrouille un compte (change mdp vers aléatoire)
    """
    user = get_object_or_404(User, pk=user)
    form_refuse = LockAccountForm(request.POST)
    if form_refuse.is_valid():
        if apps.is_installed('mri_db_logger'):
            ActionsLogs.objects.create(action='Account Lock',
                                       user=request.user,
                                       infos='Locked account username: {}'.format(
                                           user.username))
        change_user_passwd(user, uuid4().hex)
        mail_with_template('mails/account_locked.html', user.email,
                           raison=form_refuse.cleaned_data['raison'],
                           request=request)
        user.locked = True
        user.save()
    return redirect(reverse('manage_account', kwargs={'user': user.pk}))

class ManageMyAccount(LoginRequiredMixin, View):
    template_name = 'manage_user/manage_account.html'
    email_changed_template = 'manage_user/email_changed_message.html'
    http_method_names = ['get', 'post']

    def get(self, request, *args, **kwargs):
        sync_one_uid(request.user.username)
        manage_user_form = ManageMyAccountForm(instance=request.user, request=request)
        return render(request, self.template_name, {
            "manage_user_form": manage_user_form})

    def post(self, request, *args, **kwargs):
        before = model_to_dict(request.user, ['username', 'first_name', 'last_name', 'email', 'account_type'])
        manage_user_form = ManageMyAccountForm(request.POST, instance=request.user, request=request)
        saved = False
        if manage_user_form.has_changed() and manage_user_form.is_valid():
            saved = True
            manage_user_form.save()
            after = model_to_dict(request.user, ['username', 'first_name', 'last_name', 'email', 'account_type'])
            if apps.is_installed('mri_db_logger'):
                ActionsLogs.objects.create(action='Manage my account',
                                           user=request.user,
                                           infos={'before': before,
                                                  'after': after})
            if 'email' in manage_user_form.changed_data and self.request.user.account_type != 'administrator':
                waiting_validation_mail = WaitingValidationMails.objects.get(user=request.user)
                retour = mail_with_template('mails/mail_confirmation.html', waiting_validation_mail.new_email,
                                            request=request, newuser=waiting_validation_mail)
                if not retour:
                    waiting_validation_mail.delete()
                    if apps.is_installed('mri_db_logger'):
                        ActionsLogs.objects.create(action='Mail error',
                                                   user=request.user,
                                                   infos='Erreur SMTP au changement de mail')
                    messages.error(request, _('There was an error with this email'))
                    return render(request, self.template_name, {
                        "manage_user_form": manage_user_form,
                        "saved": saved})
                return render(request, self.email_changed_template)
        return render(request, self.template_name, {
            "manage_user_form": manage_user_form,
            "saved": saved})


class ChangeMyPassword(LoginRequiredMixin, View):
    template_name = 'manage_user/change_password.html'
    http_method_names = ['get', 'post']

    def get(self, request, *args, **kwargs):
        password_change_form = PasswordChangeForm(user=request.user)
        return render(request, self.template_name, {
            "password_change_form": password_change_form})

    def post(self, request, *args, **kwargs):
        password_change_form = PasswordChangeForm(user=request.user,
                                                  data=request.POST)
        saved = False
        if password_change_form.has_changed() and password_change_form.is_valid():
            if apps.is_installed('mri_db_logger'):
                ActionsLogs.objects.create(action='Change password',
                                           user=request.user,
                                           infos={})
            saved = True
            password_change_form.save()
        return render(request, self.template_name, {
            "password_change_form": password_change_form,
            "saved": saved})


class ForgottentPassword(View):
    template_name_form = 'password_recovery/request_link.html'
    template_name_succes = 'password_recovery/request_link_succes.html'
    http_method_names = ['get', 'post']

    def get(self, request):
        request_link_form = ChangePasswdLinkForm()
        return render(request, self.template_name_form, {
            "request_link_form": request_link_form})

    def post(self, request):
        request_link_form = ChangePasswdLinkForm(data=request.POST)
        if request_link_form.is_valid():
            users = User.objects.filter(email=request_link_form.cleaned_data['email'])
            if len(users):
                for user in users:
                    ChangePasswdLink.objects.filter(user=user).delete()
                    new_link = ChangePasswdLink(user=user)
                    new_link.save()
                    mail_with_template('mails/reset_passwd_link.html',
                                       user.email,
                                       request=request,
                                       new_link=new_link,
                                       validity=settings.RESET_LINK_PWD_HOUR_VALIDITY)
                    if apps.is_installed('mri_db_logger'):
                        infos = model_to_dict(user, ['username', 'first_name', 'last_name', 'email', 'account_type'])
                        ActionsLogs.objects.create(action='Forgotten password link send',
                                                   infos=infos)
            return render(request, self.template_name_succes)
        return render(request, self.template_name_form, {
            "request_link_form": request_link_form})


class ChangePasswordWithLink(View):
    template_name = 'manage_user/change_password.html'
    http_method_names = ['get', 'post']

    def get(self, request, *args, **kwargs):
        try:
            ChangePasswdLink.objects.get(link_uuid=kwargs['uuid'])
        except ChangePasswdLink.DoesNotExist:
            return render(request, 'manage_user/change_link_not_exists.html')
        password_change_form = SetPasswordForm(user=request.user)
        return render(request, self.template_name, {
            "password_change_form": password_change_form})

    def post(self, request, *args, **kwargs):
        try:
            link = ChangePasswdLink.objects.get(link_uuid=kwargs['uuid'])
        except ChangePasswdLink.DoesNotExist:
            return render(request, 'manage_user/change_link_not_exists.html')
        password_change_form = SetPasswordForm(user=link.user,
                                               data=request.POST)
        saved = False
        if password_change_form.has_changed() and password_change_form.is_valid():
            saved = True
            password_change_form.save()
            ChangePasswdLink.objects.filter(link_uuid=kwargs['uuid']).delete()
            if apps.is_installed('mri_db_logger'):
                infos = model_to_dict(link.user, ['username', 'first_name', 'last_name', 'email', 'account_type'])
                ActionsLogs.objects.create(action='Password changed with forgotten pass link',
                                           infos=infos)
        return render(request, self.template_name, {
            "password_change_form": password_change_form,
            "saved": saved})

class EditMail(LoginRequiredMixin, AdministratorMixin, View):
    template_name = 'edit_mails/edit_mail.html'
    http_method_names = ['get', 'post']

    def get(self, request, *args, **kwargs):
        if 'delete draft' in request.GET:
            MailsModels.objects.get(mail_name='register_accept',
                                    test_for=request.user).delete()
            if apps.is_installed('mri_db_logger'):
                ActionsLogs.objects.create(action='Mail draft deleted by user',
                                           user=request.user,
                                           infos={})
        try:
            mail_model = MailsModels.objects.get(mail_name='register_accept',
                                                 test_for=request.user)
        except:
            mail_model = get_object_or_404(MailsModels, mail_name='register_accept',
                                           test_for=None)
        form = MailsModelsForm(instance=mail_model)
        return render(request, self.template_name, {"form":form,
                                                    "draft":False if mail_model.test_for is None else True})

    def post(self, request, *args, **kwargs):
        try:
            mail_model = MailsModels.objects.get(mail_name='register_accept',
                                                 test_for=request.user)
        except:
            mail_model = get_object_or_404(MailsModels, mail_name='register_accept',
                                           test_for=None)
        form = MailsModelsForm(request.POST,  instance=mail_model)
        if form.is_valid():
            if 'test' in request.POST:
                if mail_model.test_for == None:
                    test_mail_model = form.save(commit=False)
                    test_mail_model.pk = None
                    test_mail_model.test_for = request.user
                    test_mail_model.draft = True
                    test_mail_model.save()
                else:
                    mail_model.save()
                mail_with_template('mails/register_accept.html', request.user.email,
                          request=request, newuser=request.user,
                          test_for_username=request.user.username)
                if apps.is_installed('mri_db_logger'):
                    ActionsLogs.objects.create(action='Save mail draft',
                                               user=request.user,
                                               infos={})
                messages.success(request, _('Mail sent'))
            else:
                mail_model = form.save(commit=False)
                if mail_model.test_for is not None:
                    prod_mail_model = MailsModels.objects.get(
                        mail_name='register_accept',
                        test_for=None)
                    prod_mail_model.title = mail_model.title
                    prod_mail_model.body_fr = mail_model.body_fr
                    prod_mail_model.body_en = mail_model.body_en
                    prod_mail_model.save()
                    mail_model.delete()
                else:
                    mail_model.save()
                messages.success(request, _('Mail saved'))
                if apps.is_installed('mri_db_logger'):
                    ActionsLogs.objects.create(action='Save and publish new mail version',
                                               user=request.user, infos={})
                return redirect(reverse('edit_welcome_mail'))
        return render(request, self.template_name, {"form":form,
                                                    "draft":False if mail_model.test_for is None else True})
